<?php

class topic extends cmsFrontend {
	
	protected $useOptions = true;

	public function routeAction($action_name){

		$categories = self::getForumCategoriesList();

		if(!$categories) { return $action_name; }

		foreach ($categories as $key => $value) {
			if ($value['slug'] == $action_name) {
				array_unshift($this->current_params, $action_name);
				return 'index';
			}
		}

		if (substr($action_name, -5) == '.html') {
			$action_name_end = substr($action_name, -5);
			$action_name = str_replace($action_name_end, '', $action_name);
			array_unshift($this->current_params, $action_name);
			return 'view';
		}

		return $action_name;
 
    }

	public function getForumCategoriesList($categories=false, $id=0, $level=0){

		if(!$categories) {

			$this->model->filterPermsCategory();

			$this->model->orderBy('ordering');
			$categories = $this->model->getForumCategories();

		}

		if(!$categories) { return false; }

		foreach ($categories as $key => $value) {
			
			if($value['parent_id'] == $id) {
				
				$list[$key] = $value;
				$list[$key]['level'] = $level;

				foreach ($categories as $key2 => $value2) {
					
					if($key == $value2['parent_id']) {

						$continue = self::getForumCategoriesList($categories, $key, $level+1);

						foreach ($continue as $key3 => $value3) {
							
							$list[$key3] = $value3;
							$list[$key3]['parent_title'] = $categories[$value3['parent_id']]['title'];
							$list[$key3]['parent_slug'] = $categories[$value3['parent_id']]['slug'];
							$list[$key3]['bread_crumb'] = self::categoriesBreadCrumb($categories, $value3['parent_id']);
						
						}

					}

				}

			} else {
				continue;
			}

		}

		return isset($list) ? $list : false;

	}

	public function categoriesBreadCrumb($categories, $id){

		foreach ($categories as $key => $value) {
			
			if ($value['id'] == $id) {

				$list[] = $value;

				if ($value['parent_id']) {

					$continue = self::categoriesBreadCrumb($categories, $value['parent_id']);

					foreach ($continue as $key2 => $value2) {
						
						$list[] = $value2;

					}

				}

			}

		}

		return array_reverse($list);

	}

	public function isModerator($user_id, $mods){

		if (cmsUser::isAllowed('topic', 'moderator')) { return true; }

		if (!$user_id || !$mods) { return false; }

		if (!is_array($mods)) {
			$mods = cmsModel::yamlToArray($mods);
		}

		foreach ($mods as $key => $value) {
			if ($user_id == $key) {
				return true;
				break;
			}
		}

		return false;

	}

	public function handlerForm($form, $type='thread', $param=false){

		$user = cmsUser::getInstance();

		// Получаем поля
        $content_model = cmsCore::getModel('content');
        $content_model->setTablePrefix('');
        $content_model->orderBy('ordering');
        $fields = $content_model->getContentFields('topic_threads');

    	if ($fields) {
			// Разбиваем поля по группам
			$fieldsets = cmsForm::mapFieldsToFieldsets($fields, function($field, $user) use ($type){
				if (($type == 'thread' && empty($field['is_in_item'])) || ($type == 'post' && empty($field['is_in_list']))) { return false; }
			    // проверяем что группа пользователя имеет доступ к редактированию этого поля
			    if ($field['groups_edit'] && !$user->isInGroups($field['groups_edit'])) { return false; }
			    return true;
			});
			// Добавляем поля в форму
			foreach ($fieldsets as $key => $value) {
			   	if(empty($value['title'])) { unset($fieldsets[$key]); }
			}
			if($fieldsets){
			    foreach($fieldsets as $fieldset){
			        $fid = $fieldset['title'] ? md5($fieldset['title']) : null;
			        $fieldset_id = $form->addFieldset($fieldset['title'], $fid);
			        foreach($fieldset['fields'] as $field){
			            // добавляем поле в форму
			            $form->addField($fieldset_id, $field['handler']);
			        }
			    }
			}
		}

		if ($type == 'post' || empty($param['category']['stickers'])) {
			$form->removeField(false, 'sticker');
		}

		return $form;

	}

	public function checkFlood($user, $type='thread'){

		$this->model->resetFilters();

		if ($type == 'thread') {

			$limit = cmsUser::getPermissionValue('topic', 'thread_flood_count');
			$time = cmsUser::getPermissionValue('topic', 'thread_flood_time');

			if (empty($limit) || empty($time)) { return false; }
			
			$items = $this->model->filterEqual('user_id', $user->id)->filterIsNull('thread_id')->orderBy('id', 'desc')->filterDateYounger('date_pub', $time, 'minute')->getAllThreadsCount();
		
		} else {

			$limit = cmsUser::getPermissionValue('topic', 'post_flood_count');
			$time = cmsUser::getPermissionValue('topic', 'post_flood_time');

			if (empty($limit) || empty($time)) { return false; }

			$items = $this->model->filterEqual('user_id', $user->id)->filterNotNull('thread_id')->orderBy('id', 'desc')->filterDateYounger('date_pub', $time, 'minute')->getAllPostCount();

		}

		if ($items < $limit) { return false; }

		return array('limit'=>$limit, 'time'=>$time);

	}

	public function addHead($data){

		$template = cmsTemplate::getInstance();

		if (isset($data['title'])) {
			$template->addHead('<meta property="og:title" content="'.$data['title'].'"/>');
		}

		if (isset($data['description'])) {
			$template->addHead('<meta property="og:description" content="'.string_get_meta_description($data['description']).'"/>');
		}

		if (isset($data['url'])) {
			$template->addHead('<meta property="og:url" content="'.$data['url'].'"/>');
		}

	}

	public function sendReport($thread, $text, $mods){

		$messenger = cmsCore::getController('messages');
		$user = cmsUser::getInstance();
		$user = $user->id ? '<a href="'.href_to('users', $user->id).'">'.$user->nickname.'</a>' : 'Гость';

		foreach ($mods as $key => $value) {
			$messenger->addRecipient($key);
		}

		$url = href_to('topic', 'thread', $thread['id']);

		$content = '<h3>'.LANG_TOPIC_FORUM_MODERATION.'</h3><p><b>'.LANG_TOPIC_THREAD.':</b> <a href="'.$url.'">'.$thread['title'].'</a><br /><b>'.LANG_TOPIC_REASON.':</b> '.$text.'<br /><b>'.LANG_TOPIC_SENDED.':</b> '.$user.'</p>';

		$messenger->ignoreNotifyOptions()->sendNoticePM(array(
            'content' => $content,
            'options' => array(
                'is_closeable' => true
            ),
            'actions' => array(
                'view' => array(
                    'title' => LANG_TOPIC_CLOSE_AND_REDIRECT,
                    'href' => $url
                )
            )
        ), 'send_report');

	}

	public function dateOnline($date){

		if (date('Y-m-d', strtotime($date)) == date('Y-m-d')) {
			return LANG_TOPIC_TODAY;
		} elseif (date('Y-m-d', strtotime('-1 day')) == date('Y-m-d', strtotime($date))) {
			return LANG_TOPIC_YESTERDAY;
		} else {
			return html_date($date);
		}

	}

	//Сравниваем массивы групп пользователя с установленными в настройках
	public function showUserGroups($groups=''){

		$show_groups = $this->options['show_user_groups'];

		if (empty($show_groups) || empty($groups)) { return false; }

		$groups = cmsModel::yamlToArray($groups);

		$list = array();

		foreach ($groups as $key => $value) {
			foreach ($show_groups as $key2 => $value2) {
				if ($value == $value2) { $list[] = $value; }
			}
		}

		return $list;

	}

	public function showUserRanks($user_ranks, $item){

		if(empty($user_ranks)) { return false; }

		if(!is_array($item)) {
			$arr = array('user_rating'=>$item->rating, 'user_karma'=>$item->karma, 'user_topic_thread_counter'=>$item->topic_thread_counter, 'user_topic_post_counter'=>$item->topic_post_counter, 'user_friends_count'=>$item->friends_count, 'user_is_locked'=>$item->is_locked, 'user_id'=>$item->id);
			$item = $arr;
		}

		$ranks = array();

		foreach ($user_ranks as $key => $value) {
			
			if ( !empty($value['rating']) && $item['user_rating'] < $value['rating'] ) { continue; }
			if ( !empty($value['karma']) && $item['user_karma'] < $value['karma'] ) { continue; }
			if ( !empty($value['threads']) && $item['user_topic_thread_counter'] < $value['threads'] ) { continue; }
			if ( !empty($value['posts']) && $item['user_topic_post_counter'] < $value['posts'] ) { continue; }
			if ( !empty($value['friends']) && $item['user_friends_count'] < $value['friends'] ) { continue; }
			if ( !empty($value['ban']) && !$item['user_is_locked'] ) { continue; }

			if(!empty($value['user_id'])) {
				$ids = str_replace(' ', '', $value['user_id']);
				$ids = explode(',', $ids);
				if (  !in_array($item['user_id'], $ids) ) { continue; }
			}

			$ranks[] = $value;
		
		}

		if (!$ranks) { return false; }

		$ranks = array_pop($ranks);

		return $ranks;

	}

	public function generateThreadSlug($item){

		$pattern = $this->options['thread_url_pattern'];

		if (empty($pattern)) {
			$pattern = 'thread{id}';
		}

		$pos = strpos($pattern, '{id}');

		if ($pos === false) {
			$pattern = '{id}'.$pattern;
		}

		$data = array('id'=>$item['id'], 'title'=>$item['title'], 'date'=>date('Y-m-d H:i:s'), 'category'=>$item['category_slug'], 'user_id'=>$item['user_id']);

		$slug = string_replace_keys_values($pattern, $data);
		$slug = str_replace('/', '', $slug);
		$slug = lang_slug($slug);

		return $slug;

	}

	public function showUserMedals($user_medals, $item){

		if(empty($user_medals)) { return false; }

		if(!is_array($item)) {
			$arr = array('user_rating'=>$item->rating, 'user_karma'=>$item->karma, 'user_topic_thread_counter'=>$item->topic_thread_counter, 'user_topic_post_counter'=>$item->topic_post_counter, 'user_friends_count'=>$item->friends_count, 'user_is_locked'=>$item->is_locked, 'user_id'=>$item->id);
			$item = $arr;
		}

		$medals = array();

		foreach ($user_medals as $key => $value) {
			if ( !empty($value['rating']) && $item['user_rating'] < $value['rating'] ) { continue; }
			if ( !empty($value['karma']) && $item['user_karma'] < $value['karma'] ) { continue; }
			if ( !empty($value['threads']) && $item['user_topic_thread_counter'] < $value['threads'] ) { continue; }
			if ( !empty($value['posts']) && $item['user_topic_post_counter'] < $value['posts'] ) { continue; }
			if ( !empty($value['friends']) && $item['user_friends_count'] < $value['friends'] ) { continue; }

			if(!empty($value['user_id'])) {
				$ids = str_replace(' ', '', $value['user_id']);
				$ids = explode(',', $ids);
				if (  !in_array($item['user_id'], $ids) ) { continue; }
			}
			
			$medals[] = $value;
		}

		if (!$medals) { return false; }

		return $medals;

	}

	public function rankAllowed($rank, $items, $user){

		if (!$rank) { return false; }

		$items = cmsModel::yamlToArray($items);

		if($user->is_admin || $user->is_moderator || in_array($rank['id'], $items)) {
			return true;
		}

		return false;

	}

	public function medalAllowed($medals, $items, $user){

		if (!$medals) { return false; }

		$items = cmsModel::yamlToArray($items);

		if($user->is_admin || $user->is_moderator) {
			return true;
		}

		foreach ($items as $key => $value) {
			foreach ($medals as $key2 => $value2) {
				if ($value == $value2['id']) { return true; break; }
			}
		}

		return false;

	}

	public function getSticker($stickers, $value) {

		if (empty($stickers) || empty($value)) { return false; }

		$stickers = string_parse_list($stickers);

		foreach ($stickers as $key=>$values) {
			if (!isset($values['id'])) { $values['id'] = $key; }
			if ($values['id'] == $value) {
				if (is_numeric($value)) {
					$item['bg_color'] = 'orange';
				} else {
					$item['bg_color'] = $value;
				}
				if ($value{0} == '#') {
					$item['color'] = true;
				}
				$item['title'] = $values['value'];
				return $item; break;
			}
		}

		return false;

	}
	
}
